@@ -1,6 +1,5 @@
 /* This file is part of the KDE libraries
     Copyright (C) 2001,2002 Ellis Whitehead <ellis@kde.org>
-    Copyright (C) 2013 Martin Gräßlin <mgraesslin@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -20,21 +19,36 @@
 
 #include "kglobalaccel_x11.h"
 
+#include <QWidgetList>
+
+#include "kaction.h"
 #include "globalshortcutsregistry.h"
 #include "kkeyserver.h"
 
-#include <QDebug>
+#include <kapplication.h>
+#include <kdebug.h>
 
-#include <QApplication>
+#include <QtCore/QRegExp>
 #include <QWidget>
-#include <QX11Info>
+#include <QtCore/QMetaClassInfo>
+#include <QMenu>
 
+#include <kxerrorhandler.h>
+
+#include <X11/X.h>
+#include <X11/Xlib.h>
+#include <X11/Xutil.h>
 #include <X11/keysym.h>
+#include <fixx11h.h>
 
-// xcb
-#include <xcb/xcb.h>
-#include <xcb/xcb_keysyms.h>
-
+extern "C" {
+  static int XGrabErrorHandler( Display *, XErrorEvent *e ) {
+	if ( e->error_code != BadAccess ) {
+	    kWarning() << "grabKey: got X error " << e->type << " instead of BadAccess\n";
+	}
+	return 1;
+  }
+}
 
 // g_keyModMaskXAccel
 //	mask of modifiers which can be used in shortcuts
@@ -53,80 +67,63 @@ static void calculateGrabMasks()
 			KKeyServer::modXNumLock() |
 			KKeyServer::modXScrollLock() |
 			KKeyServer::modXModeSwitch();
-	//qDebug() << "g_keyModMaskXAccel = " << g_keyModMaskXAccel
+	//kDebug() << "g_keyModMaskXAccel = " << g_keyModMaskXAccel
 	//	<< "g_keyModMaskXOnOrOff = " << g_keyModMaskXOnOrOff << endl;
 }
 
 //----------------------------------------------------
 
 KGlobalAccelImpl::KGlobalAccelImpl(GlobalShortcutsRegistry *owner)
-    : QObject(owner)
-    , m_owner(owner)
-    , m_keySymbols(Q_NULLPTR)
+	: m_owner(owner)
 {
 	calculateGrabMasks();
-    if (QX11Info::isPlatformX11()) {
-        m_keySymbols = xcb_key_symbols_alloc(QX11Info::connection());
-    }
-}
-
-KGlobalAccelImpl::~KGlobalAccelImpl()
-{
-    if (m_keySymbols) {
-        xcb_key_symbols_free(m_keySymbols);
-    }
 }
 
 bool KGlobalAccelImpl::grabKey( int keyQt, bool grab )
 {
-    if (!m_keySymbols) {
-        return false;
-    }
 	if( !keyQt ) {
-        qDebug() << "Tried to grab key with null code.";
+        kDebug() << "Tried to grab key with null code.";
 		return false;
 	}
 
+	int keyCodeX;
 	uint keyModX;
-	xcb_keysym_t keySymX;
+	uint keySymX;
 
 	// Resolve the modifier
 	if( !KKeyServer::keyQtToModX(keyQt, &keyModX) ) {
-		qDebug() << "keyQt (0x" << hex << keyQt << ") failed to resolve to x11 modifier";
+		kDebug() << "keyQt (0x" << hex << keyQt << ") failed to resolve to x11 modifier";
 		return false;
 	}
 
 	// Resolve the X symbol
 	if( !KKeyServer::keyQtToSymX(keyQt, (int *)&keySymX) ) {
-		qDebug() << "keyQt (0x" << hex << keyQt << ") failed to resolve to x11 keycode";
+		kDebug() << "keyQt (0x" << hex << keyQt << ") failed to resolve to x11 keycode";
 		return false;
 	}
 
-    xcb_keycode_t *keyCodes = xcb_key_symbols_get_keycode(m_keySymbols, keySymX);
-    if (!keyCodes) {
-        return false;
-    }
-    xcb_keycode_t keyCodeX = keyCodes[0];
-    free(keyCodes);
+	keyCodeX = XKeysymToKeycode( QX11Info::display(), keySymX );
 
 	// Check if shift needs to be added to the grab since KKeySequenceWidget
 	// can remove shift for some keys. (all the %&* and such)
 	if( !(keyQt & Qt::SHIFT) &&
 	    !KKeyServer::isShiftAsModifierAllowed( keyQt ) &&
-	    keySymX != xcb_key_symbols_get_keysym(m_keySymbols, keyCodeX, 0) &&
-	    keySymX == xcb_key_symbols_get_keysym(m_keySymbols, keyCodeX, 1) )
+	    keySymX != XKeycodeToKeysym( QX11Info::display(), keyCodeX, 0 ) &&
+	    keySymX == XKeycodeToKeysym( QX11Info::display(), keyCodeX, 1 ) )
 	{
-		qDebug() << "adding shift to the grab";
+		kDebug() << "adding shift to the grab";
 		keyModX |= KKeyServer::modXShift();
 	}
 
 	keyModX &= g_keyModMaskXAccel; // Get rid of any non-relevant bits in mod
 
 	if( !keyCodeX ) {
-		qDebug() << "keyQt (0x" << hex << keyQt << ") was resolved to x11 keycode 0";
+		kDebug() << "keyQt (0x" << hex << keyQt << ") was resolved to x11 keycode 0";
 		return false;
 	}
 
+	KXErrorHandler handler( XGrabErrorHandler );
+
 	// We'll have to grab 8 key modifier combinations in order to cover all
 	//  combinations of CapsLock, NumLock, ScrollLock.
 	// Does anyone with more X-savvy know how to set a mask on QX11Info::appRootWindow so that
@@ -136,35 +133,27 @@ bool KGlobalAccelImpl::grabKey( int keyQt, bool grab )
 	QString sDebug = QString("\tcode: 0x%1 state: 0x%2 | ").arg(keyCodeX,0,16).arg(keyModX,0,16);
 #endif
 	uint keyModMaskX = ~g_keyModMaskXOnOrOff;
-        QVector<xcb_void_cookie_t> cookies;
 	for( uint irrelevantBitsMask = 0; irrelevantBitsMask <= 0xff; irrelevantBitsMask++ ) {
 		if( (irrelevantBitsMask & keyModMaskX) == 0 ) {
 #ifndef NDEBUG
 			sDebug += QString("0x%3, ").arg(irrelevantBitsMask, 0, 16);
 #endif
 			if( grab )
-                            cookies << xcb_grab_key_checked(QX11Info::connection(), true,
-                                                            QX11Info::appRootWindow(), keyModX | irrelevantBitsMask,
-                                                            keyCodeX, XCB_GRAB_MODE_ASYNC, XCB_GRAB_MODE_SYNC);
+				XGrabKey( QX11Info::display(), keyCodeX, keyModX | irrelevantBitsMask,
+					QX11Info::appRootWindow(), True, GrabModeAsync, GrabModeSync );
 			else
-                            cookies << xcb_ungrab_key_checked(QX11Info::connection(), keyCodeX,
-                                                              QX11Info::appRootWindow(), keyModX | irrelevantBitsMask);
+				XUngrabKey( QX11Info::display(), keyCodeX, keyModX | irrelevantBitsMask, QX11Info::appRootWindow() );
 		}
 	}
 
 	bool failed = false;
 	if( grab ) {
-            for (int i = 0; i < cookies.size(); ++i) {
-                QScopedPointer<xcb_generic_error_t, QScopedPointerPodDeleter> error(xcb_request_check(QX11Info::connection(), cookies.at(i)));
-                if (!error.isNull()) {
-                    failed = true;
-                }
-            }
+		failed = handler.error( true ); // sync now
 		if( failed ) {
-			qDebug() << "grab failed!\n";
+			kDebug() << "grab failed!\n";
 			for( uint m = 0; m <= 0xff; m++ ) {
 				if(( m & keyModMaskX ) == 0 )
-                                    xcb_ungrab_key(QX11Info::connection(), keyCodeX, QX11Info::appRootWindow(), keyModX | m);
+					XUngrabKey( QX11Info::display(), keyCodeX, keyModX | m, QX11Info::appRootWindow() );
 			}
 		}
 	}
@@ -172,31 +161,27 @@ bool KGlobalAccelImpl::grabKey( int keyQt, bool grab )
 	return !failed;
 }
 
-bool KGlobalAccelImpl::nativeEventFilter(const QByteArray &eventType, void *message, long *)
+bool KGlobalAccelImpl::x11Event( XEvent* event )
 {
-    if (eventType != QByteArrayLiteral("xcb_generic_event_t")) {
-        return false;
-    }
-    xcb_generic_event_t *event = reinterpret_cast<xcb_generic_event_t*>(message);
-    const uint8_t responseType = event->response_type & ~0x80;
-    switch (responseType) {
-        case XCB_MAPPING_NOTIFY:
-            qDebug() << "Got XMappingNotify event";
-            xcb_refresh_keyboard_mapping(m_keySymbols, reinterpret_cast<xcb_mapping_notify_event_t*>(event));
-            x11MappingNotify();
-            return true;
+	switch( event->type ) {
 
-        case XCB_KEY_PRESS:
-            qDebug() << "Got XKeyPress event";
-            return x11KeyPress(reinterpret_cast<xcb_key_press_event_t*>(event));
+		case MappingNotify:
+			kDebug() << "Got XMappingNotify event";
+			XRefreshKeyboardMapping(&event->xmapping);
+			x11MappingNotify();
+			return true;
 
-        default:
-            // We get all XEvents. Just ignore them.
-            return false;
-    }
+		case XKeyPress:
+			kDebug() << "Got XKeyPress event";
+			return x11KeyPress(event);
 
-    Q_ASSERT(false);
-    return false;
+		default:
+			// We get all XEvents. Just ignore them.
+			return false;
+	}
+
+	Q_ASSERT(false);
+	return false;
 }
 
 void KGlobalAccelImpl::x11MappingNotify()
@@ -217,27 +202,29 @@ void KGlobalAccelImpl::x11MappingNotify()
 	m_owner->grabKeys();
 }
 
-bool KGlobalAccelImpl::x11KeyPress(xcb_key_press_event_t *pEvent)
+
+bool KGlobalAccelImpl::x11KeyPress( const XEvent *pEvent )
 {
 	if (QWidget::keyboardGrabber() || QApplication::activePopupWidget()) {
-		qWarning() << "kglobalacceld should be popup and keyboard grabbing free!";
+		kWarning() << "kglobalacceld should be popup and keyboard grabbing free!";
 	}
 
 	// Keyboard needs to be ungrabed after XGrabKey() activates the grab,
 	// otherwise it becomes frozen.
-    xcb_connection_t *c = QX11Info::connection();
-    xcb_ungrab_keyboard(c, XCB_TIME_CURRENT_TIME);
-    xcb_flush(c);
+	XUngrabKeyboard( QX11Info::display(), CurrentTime );
+	XFlush( QX11Info::display()); // avoid X(?) bug
 
-    xcb_keycode_t keyCodeX = pEvent->detail;
-    uint16_t keyModX = pEvent->state & (g_keyModMaskXAccel | KKeyServer::MODE_SWITCH);
+	uchar keyCodeX = pEvent->xkey.keycode;
+	uint keyModX = pEvent->xkey.state & (g_keyModMaskXAccel | KKeyServer::MODE_SWITCH);
 
-    xcb_keysym_t keySymX = xcb_key_press_lookup_keysym(m_keySymbols, pEvent, 0);
+	KeySym keySym;
+	XLookupString( (XKeyEvent*) pEvent, 0, 0, &keySym, 0 );
+	uint keySymX = (uint)keySym;
 
 	// If numlock is active and a keypad key is pressed, XOR the SHIFT state.
 	//  e.g., KP_4 => Shift+KP_Left, and Shift+KP_4 => KP_Left.
-    if (pEvent->state & KKeyServer::modXNumLock()) {
-        xcb_keysym_t sym = xcb_key_symbols_get_keysym(m_keySymbols, keyCodeX, 0);
+	if( pEvent->xkey.state & KKeyServer::modXNumLock() ) {
+		uint sym = XKeycodeToKeysym( QX11Info::display(), keyCodeX, 0 );
 		// If this is a keypad key,
 		if( sym >= XK_KP_Space && sym <= XK_KP_9 ) {
 			switch( sym ) {
@@ -262,7 +249,7 @@ bool KGlobalAccelImpl::x11KeyPress(xcb_key_press_event_t *pEvent)
 	KKeyServer::modXToQt(keyModX, &keyModQt);
 
 	if( keyModQt & Qt::SHIFT && !KKeyServer::isShiftAsModifierAllowed( keyCodeQt ) ) {
-		qDebug() << "removing shift modifier";
+		kDebug() << "removing shift modifier";
 		keyModQt &= ~Qt::SHIFT;
 	}
 
@@ -274,11 +261,11 @@ bool KGlobalAccelImpl::x11KeyPress(xcb_key_press_event_t *pEvent)
 
 void KGlobalAccelImpl::setEnabled( bool enable )
 {
-    if (enable && qApp->platformName() == QByteArrayLiteral("xcb")) {
-        qApp->installNativeEventFilter(this);
-    } else {
-        qApp->removeNativeEventFilter(this);
-    }
+	if (enable) {
+		kapp->installX11EventFilter( this );
+	} else
+		kapp->removeX11EventFilter( this );
 }
 
+
 #include "kglobalaccel_x11.moc"
